==> tuxnes-dc-0.1.tgz: tuxnes-dc-0.1/dc-sound.c <==
#include "dc-aica/aica_fw.h"
#include "dc-glue.h" // for malloc
#include <kos.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include "globals.h"
#include "sound.h"

/* This should really go in an include file */
volatile unsigned long *snd_iface = (unsigned long*)0xa080ffc0;
#define SND_STATUS	0
#define SND_MONO	1
#define SND_SMP		2
#define SND_SMP_WHERE	3
#define SND_SMP_SIZE	4
#define SND_SMP_HZ	5
#define SND_SMP_8BIT	6
#define SND_SMP_LOOP	7
#define SND_SMP_POS	8

#define SAMPLES 734
#define BUF_LEN (SAMPLES * bytes_per_sample)
#define BUFFERS 4
#define AICA_SAMPLE_MEM 0x10000

static int audiofd = -1;
static int buf_size;

unsigned char *audio_buffer1;

int
InitAudioDC(int argc, char **argv)
{
	char printf_buf[256];
	int i;
//	if (verbose) fprintf(stderr, "DC sound\n");

	sample_format_number = sample_format->number;
	switch (sample_format_number) {
		case AFMT_U16_LE:
		case AFMT_U16_BE:
		case AFMT_S16_LE:
		case AFMT_S16_BE:
			bytes_per_sample = 2;
	}
	switch (sample_format_number) {
		case AFMT_S8:
		case AFMT_S16_LE:
		case AFMT_S16_BE:
			signed_samples = 1;
	}

//	spu_init();
	spu_disable();
	spu_memload(0, play, sizeof(play));
	spu_enable();

	// Wait for ARM program to initialize.
	while (snd_iface[SND_STATUS] <= 3);

	audiofd = 1;

	InitSoundParams(argc);

	audio_buffer = malloc( BUF_LEN * BUFFERS ); /* 2940 samples 5880 bytes */
	audio_buffer1 = audio_buffer;

	if ( audio_buffer == NULL ) {
	//      fprintf( stderr, "Error allocating sound buffer." );
		audiofd = -1;
		return 1;
	}
	// FIXME Set to 0x50 to hear clicking
	memset( audio_buffer, 0x00, BUF_LEN * BUFFERS);

	/* Start AICA looping. */
	snd_iface[SND_SMP_WHERE] = AICA_SAMPLE_MEM;
	snd_iface[SND_SMP_SIZE]  = SAMPLES * BUFFERS;
	/* Synchronize (approximately) with the TV refresh */
/*	snd_iface[SND_SMP_HZ]    = 43992;  // For 735 samples/buffer */
	snd_iface[SND_SMP_HZ]    = 43928;  // For 734 samples/buffer
	snd_iface[SND_SMP_8BIT]  = (bytes_per_sample == 1) ? 1 : 0;
	snd_iface[SND_SMP_LOOP]  = 1;
	snd_iface[SND_SMP]       = 1;      // Go!
	printf("looping %d %d-bit samples\n", SAMPLES * BUFFERS, bytes_per_sample*8);

  return 0;
}

void
UpdateAudioDC(void) /* called freq times a sec */
{
    static  unsigned char *buf = NULL;
	static unsigned int p = 0;
	static unsigned int dest = AICA_SAMPLE_MEM;
	static unsigned int last_sample = 0;
	static unsigned int last_pos = 0;
	unsigned int this_pos = 0, buffer = 0;
	int diff_pos = 0;
	int i, j, k;
	char printf_buf[256];

    ProgressiveSoundUpdate( 1 );
    if ( audiofd < 0 ) return;

/* tuxnes's samples are always 1 byte
 * regardless of 8-bit or 16-bit sample size.
 * (samples_per_vsync is doubled when 16-bit)
 */

/* libdream has an off-by-one error when len % 4 == 0 */

	this_pos = snd_iface[SND_SMP_POS];
	buffer = this_pos / SAMPLES;

    /* diff_pos is not currently necessary. */
#if 0
	diff_pos = this_pos - last_pos;

	/* Wrap around if necessary. */
	if (diff_pos < 0) {
		diff_pos += SAMPLES * 4;
	}
#endif

	/* We can enter the same buffer twice in a row if frames are
	   skipped.  In this case, advance to the next buffer. */
	/* Note: This requires a consistent framerate, and
	   only works with frameskip == 0 or 1. */
	if ((buffer == last_pos / SAMPLES)
        /* && (diff_pos < SAMPLES - 100) */ ) {
		buffer++;
	}

	/* Write to the next buffer (relative to the one we're in). */
	buffer++;
	buffer &= 0x03;

#if 0
	if (total_temp_ctr != 0) {
		printf("%d(%d)", buffer, this_pos);
	}
#endif

	dest = AICA_SAMPLE_MEM + buffer * BUF_LEN;
	spu_memload ( dest, audio_buffer1, BUF_LEN);
		
	p++;
	last_pos = this_pos;

}

int ShutdownAudioDC(void)
{
	return spu_shutdown();
}

==> tuxnes-dc-0.2.tgz: tuxnes-dc-0.2/dc-sound.c <==
#include "dc-aica/aica_fw.h"
#include "dc-glue.h" // for malloc
#include <kos.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include "globals.h"
#include "sound.h"

/* This should really go in an include file */
volatile unsigned long *snd_iface = (unsigned long*)0xa080ffc0;
#define SND_STATUS	0
#define SND_MONO	1
#define SND_SMP		2
#define SND_SMP_WHERE	3
#define SND_SMP_SIZE	4
#define SND_SMP_HZ	5
#define SND_SMP_8BIT	6
#define SND_SMP_LOOP	7
#define SND_SMP_POS	8

#define SAMPLES 735
#define BUF_LEN (SAMPLES * bytes_per_sample)
#define BUFFERS 4
#define AICA_SAMPLE_MEM 0x10000

static int audiofd = -1;
static int buf_size;

unsigned char *audio_buffer1;

int
InitAudioDC(int argc, char **argv)
{
	char printf_buf[256];
	int i;
//	if (verbose) fprintf(stderr, "DC sound\n");

	sample_format_number = sample_format->number;
	bytes_per_sample = 1;
	signed_samples   = 0;

	switch (sample_format_number) {
		case AFMT_U16_LE:
		case AFMT_U16_BE:
		case AFMT_S16_LE:
		case AFMT_S16_BE:
			bytes_per_sample = 2;
	}
	switch (sample_format_number) {
		case AFMT_S8:
		case AFMT_S16_LE:
		case AFMT_S16_BE:
			signed_samples = 1;
	}

//	spu_init();
	spu_disable();
	spu_memload(0, play, sizeof(play));
	spu_enable();

	// Wait for ARM program to initialize.
	while (snd_iface[SND_STATUS] <= 3);

	audiofd = 1;

	InitSoundParams(argc);

	/* FIXME !! Isn't audio_buffer just 735 samples long? */
	audio_buffer = malloc( BUF_LEN * BUFFERS ); /* 2940 samples 5880 bytes */
	audio_buffer1 = audio_buffer;

	if ( audio_buffer == NULL ) {
	//      fprintf( stderr, "Error allocating sound buffer." );
		audiofd = -1;
		return 1;
	}
	// FIXME Set to 0x50 to hear clicking
	memset( audio_buffer, 0x00, BUF_LEN * BUFFERS);

	/* Start AICA looping. */
	snd_iface[SND_SMP_WHERE] = AICA_SAMPLE_MEM;
	snd_iface[SND_SMP_SIZE]  = SAMPLES * BUFFERS;
	/* Synchronize (approximately) with the TV refresh */
//	snd_iface[SND_SMP_HZ]    = 43992;  // For 735 samples/buffer
/*	snd_iface[SND_SMP_HZ]    = 43928;  // For 734 samples/buffer */
	snd_iface[SND_SMP_HZ]    = 44100;
	snd_iface[SND_SMP_8BIT]  = (bytes_per_sample == 1) ? 1 : 0;
	snd_iface[SND_SMP_LOOP]  = 1;
	snd_iface[SND_SMP]       = 1;      // Go!
	printf("looping %d %d-bit samples\n", SAMPLES * BUFFERS, bytes_per_sample*8);

  return 0;
}

void
UpdateAudioDC(void) /* called freq times a sec */
{
	/* FIXME these static variables should probably be global so
	 * we can init them every time we start a new game */
	static unsigned int last_pos = 0;
	unsigned int this_pos = 0;
	unsigned int diff_pos;
	unsigned int dest;

/* tuxnes's samples are always 1 byte
 * regardless of 8-bit or 16-bit sample size.
 * (samples_per_vsync is doubled when 16-bit)
 */

	this_pos = snd_iface[SND_SMP_POS];
	/* Align on a 4-byte boundary. */
	this_pos &= ~3;

	if (last_pos < this_pos) {
		diff_pos = this_pos - last_pos;
	} else {
		diff_pos = (SAMPLES*BUFFERS) - last_pos + this_pos;
	}

	/* Generate only as many samples as the hardware played
	 * since last update. */
    ProgressiveSoundUpdate( diff_pos * bytes_per_sample ); /* tuxnes num_samples are double when 16-bit (!) */
    if ( audiofd < 0 ) return;

	dest = AICA_SAMPLE_MEM + last_pos*bytes_per_sample;
	/* spu_memload's accepts a byte offset destination,
	 * but it'll crash unless it's on a 4-byte boundary. */
	if (last_pos < this_pos) {
		spu_memload ( dest, audio_buffer1, diff_pos * bytes_per_sample );
	} else {
		spu_memload ( dest, audio_buffer1, (SAMPLES*BUFFERS - last_pos)*bytes_per_sample);
		spu_memload ( AICA_SAMPLE_MEM, audio_buffer1 + (SAMPLES*BUFFERS - last_pos)*bytes_per_sample, this_pos*bytes_per_sample);
		/* adjust for wrap-around */
	}
		
	last_pos = this_pos;

}

int ShutdownAudioDC(void)
{
	/* This also clears out the AICA's memory. */
	return spu_shutdown();
}
